#!/usr/bin/perl -w

die;

#
# dbfilesplit.pm
# Copyright (C) 2000-2007 by John Heidemann <johnh@isi.edu>
# $Id: e537c968b2d3ace69b5d0995ee586156e0c2c24c $
#
# This program is distributed under terms of the GNU general
# public license, version 2.  See the file COPYING
# in $dblibdir for details.
#

package Jdb::Filter::dbfilesplit;

=head1 NAME
=head1 SYNOPSIS
=head1 DESCRIPTION

sub usage {
    print <<END;
usage: $0 [-p prefix] [-s suffix]

Splits the input into multiple files
'filename.0',, 'filename.1', ..., using lines starting
with "#h" or "#l" as a marker-separator.
By default files are called stdin.0.jdb, stdin.1.jdb, etc.

=head1 OPTIONS

=over

=back


-p prefix	the prefix on files (default "stdin.")
-s suffix	the suffix on files (default ".jdb"

If you specify your own prefix and suffix, you need to include dots (if
you want them).

=for comment
begin_standard_jdb_options

This module also supports the standard jdb options:

=over

=item B<-d>

Enable debugging output.

=item B<-i> or B<--input> InputSource

Read from InputSource, typically a file name, or C<-> for standard input,
or (if in Perl) a IO::Handle, Jdb::IO or Jdb::BoundedQueue objects.

=item B<-o> or B<--output> OutputDestination

Write to OutputDestination, typically a file name, or C<-> for standard output,
or (if in Perl) a IO::Handle, Jdb::IO or Jdb::BoundedQueue objects.

=item B<--autorun> or B<--noautorun>

By default, programs process automatically,
but Jdb::Filter objects in Perl do not run until you invoke
the run() method.
The C<--(no)autorun> option controls that behavior within Perl.

=item B<--help>

Show help.

=item B<--man>

Show full manual.

=back

=for comment
end_standard_jdb_options


=head1 SAMPLE USAGE

=head2 Input:

#h sid cid
1 10
2 11
1 12
2 12
#h cid cname
10 pascal
11 numanal
12 os

=head2 Command:

cat DATA/filesplit.jdb | dbfilesplit -p classinfo.

=head2 Output:

(nothing to stdout)

File classinfo.0.jdb:
#h sid cid
1 10
2 11
1 12
2 12

File classinfo.1.jdb:
#h cid cname
10 pascal
11 numanal
12 os

This program is based on code contributed from Pavlin Radoslavov.

END
# '
    exit 1;


=head1 SEE ALSO

L<Jdb>.


=head1 CLASS FUNCTIONS

=cut

@ISA = qw(Jdb::Filter);
$VERSION = 2.0;

use strict;
use Pod::Usage;
use Carp;

use Jdb::Filter;
use Jdb::IO::Reader;
use Jdb::IO::Writer;


=head2 new

    $filter = new Jdb::Filter::dbfilesplit(@arguments);

Create a new dbfilesplit object, taking command-line arguments.

=cut

sub new ($@) {
    my $class = shift @_;
    my $self = $class->SUPER::new(@_);
    bless $self, $class;
    $self->set_defaults;
    $self->parse_options(@_);
    $self->SUPER::post_new();
    return $self;
}


=head2 set_defaults

    $filter->set_defaults();

Internal: set up defaults.

=cut

sub set_defaults ($) {
    my($self) = @_;
    $self->SUPER::set_defaults();
}

=head2 parse_options

    $filter->parse_options(@ARGV);

Internal: parse command-line arguments.

=cut

sub parse_options ($@) {
    my $self = shift @_;

    my(@argv) = @_;
    $self->get_options(
	\@argv,
 	'help|?' => sub { pod2usage(1); },
	'man' => sub { pod2usage(-verbose => 2); },
	'autorun!' => \$self->{_autorun},
	'd|debug+' => \$self->{_debug},
	'i|input=s' => sub { $self->parse_io_option('input', @_); },
	'log!' => \$self->{_logprog},
	'o|output=s' => sub { $self->parse_io_option('output', @_); },
	) or pod2usage(2);
    push (@{$self->{_argv}}, @argv);
}

=head2 setup

    $filter->setup();

Internal: setup, parse headers.

=cut

sub setup ($) {
    my($self) = @_;

    $self->finish_io_option('input', -comment_handler => $self->create_pass_comments_sub);
    my $read_fastpath_sub = $self->{_in}->fastpath_sub();

    $self->finish_io_option('output', -clone => $self->{_in});
    my $write_fastpath_sub = $self->{_out}->fastpath_sub();
}

=head2 run

    $filter->run();

Internal: run over each rows.

=cut
sub run ($) {
    my($self) = @_;
    &{$self->{_loop_sub}}();
}


use Jdb::Old;
use Jdb::DbGetopt;

my(@orig_argv) = @ARGV;
my($prog) = &progname;
my($debug);
my($prefix, $suffix) = qw(stdin. .jdb);

my($dbopts) = new Jdb::DbGetopt("dp:s:?", \@ARGV);
my($ch);
while ($dbopts->getopt) {
    $ch = $dbopts->opt;
    if ($ch eq 'd') {
	$debug++;
    } elsif ($ch eq 'p') {
	$prefix = $dbopts->optarg;
    } elsif ($ch eq 's') {
	$suffix = $dbopts->optarg;
    } else {
	&usage;
    };
};
# &usage if ($#ARGV < -1);

my($count) = -1;

while (<STDIN>) {
    if (/^$headertag_regexp/) {
	&process_header($_);
	&close_file;
	&open_file;
	&write_header();
    } else {
	die "$0: lines before first tag\n" if ($count < 0);
	print $_;
    };
};
&close_file;

exit 0;


sub open_file {
    $count++;
    open(STDOUT, ">$prefix$count$suffix") || die "$0: cannot write $prefix$count$suffix\n";
}

sub close_file {
    return if ($count < 0);
    # die "$0: close_file before open_file\n" if ($count < 0);
    print "#  | $prog ", join(" ", @orig_argv), "\n";
    close STDOUT;
}





=head2 finish

    $filter->finish();

Internal: write trailer.

=cut
sub finish ($) {
    my($self) = @_;

    $self->SUPER::finish();
}

=head1 AUTHOR and COPYRIGHT

Copyright (C) 1991-2007 by John Heidemann <johnh@isi.edu>

This program is distributed under terms of the GNU general
public license, version 2.  See the file COPYING
with the distribution for details.

=cut

1;
